"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs-extra");
const recast_1 = require("recast");
function generator({ models, storeGenerator, projectType, rootDir, iceTemp }) {
    let source = '';
    const names = [];
    let imports = models.map(model => {
        const { name, relativePath } = model;
        names.push(name);
        return `import ${name} from '${relativePath}'`;
    });
    if (typeof storeGenerator.appendImports === 'function') {
        imports = storeGenerator.appendImports(imports);
    }
    const config = generatorConfig(rootDir, iceTemp, storeGenerator.configName || 'store', projectType);
    if (config.imports.length > 0) {
        imports = imports.concat(config.imports);
    }
    source += imports.join(';\n') + ';\n\n';
    // add config source
    source += config.source;
    source += `export const models = {
    ${names.length > 0 ? names.join(',\n  ') : ''}
  };\n\n`;
    if (typeof storeGenerator.appendSource === 'function') {
        source += storeGenerator.appendSource(projectType, config.hasConfig);
    }
    return source;
}
exports.generator = generator;
const appReg = /^(App|app)\.(ts|js|tsx|jsx)$/;
/**
 * 从 app.js 中读取 plugin config
 * @param rootDir
 * @param iceTemp
 * @param storeType
 * @param projectType
 */
function generatorConfig(rootDir, iceTemp, configName, projectType) {
    const dirs = fs.readdirSync(path.join(rootDir, './src'));
    const files = dirs.filter(dir => appReg.test(dir));
    const imports = [];
    let source = '';
    let hasConfig = false;
    if (files.length > 0) {
        const type = checkConfig(path.join(rootDir, 'src', files[0]), configName);
        if (!type) {
            return { source, imports: [], hasConfig: false };
        }
        hasConfig = true;
        const configPath = path.relative(iceTemp, `${rootDir}/src/${files[0].split('.')[0]}`);
        if (type === 'exportDefault') {
            imports.push(`import appConfig from '${configPath}'`);
            if (projectType === 'ts') {
                source = `interface IAppConfig {
          ${configName}?: any;
        }

        const config = (<IAppConfig>appConfig || {}).${configName} || {};`;
                source += '\n\n';
            }
            else {
                source = `const config = (appConfig || {}).${configName} || {};`;
            }
        }
        else {
            imports.push(`import { ${configName} as config } from '${configPath}'`);
        }
    }
    return { source, imports, hasConfig };
}
exports.generatorConfig = generatorConfig;
// 检查 app.ts 文件导出
function checkConfig(file, configName) {
    try {
        const code = fs.readFileSync(file, 'utf8');
        const ast = recast_1.parse(code, {
            parser: require("recast/parsers/babel"),
        });
        let ExportDefaultDeclaration = [];
        let ExportNamedDeclaration = [];
        let ExpressionStatement = [];
        ast.program.body.forEach((body) => {
            let keys;
            switch (body.type) {
                case 'ExportDefaultDeclaration':
                    const { properties } = body.declaration;
                    keys = properties.map((propertie) => propertie.key.name);
                    ExportDefaultDeclaration = ExportDefaultDeclaration.concat(keys);
                    break;
                case 'ExportNamedDeclaration':
                    const { declarations } = body.declaration;
                    keys = declarations.map((declaration) => declaration.id.name);
                    ExportNamedDeclaration = ExportNamedDeclaration.concat(keys);
                    break;
                case 'ExpressionStatement':
                    const { left, right } = body.expression;
                    const { property: { name } } = left;
                    if (name === 'exports') {
                        const { properties } = right;
                        keys = properties.map((propertie) => propertie.key.name);
                    }
                    else {
                        keys = [name];
                    }
                    ExpressionStatement = ExpressionStatement.concat(keys);
                    break;
                default:
                    break;
            }
        });
        if (ExportDefaultDeclaration.length > 0 && ExportDefaultDeclaration.includes(configName)) {
            return 'exportDefault';
        }
        if (ExportNamedDeclaration.length > 0 && ExportNamedDeclaration.includes(configName)) {
            return 'exportNamed';
        }
        if (ExpressionStatement.length > 0 && ExpressionStatement.includes(configName)) {
            return 'exportCommand';
        }
    }
    catch (err) { }
    return false;
}
