"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs-extra");
const chokidar = require("chokidar");
const prettier = require("prettier");
const utils_1 = require("../utils");
const stores_1 = require("../stores");
const webpack_1 = require("../alias/webpack");
const config_1 = require("../alias/config");
const getModels_1 = require("./getModels");
const generator_1 = require("./generator");
class PluginCore {
    constructor({ log, rootDir, projectType, iceTemp, command, options = {}, onGetWebpackConfig, applyMethod }) {
        const { storePath, storeType } = options;
        this.log = log;
        this.rootDir = rootDir;
        this.projectType = projectType;
        this.iceTemp = iceTemp ? iceTemp : path.join(rootDir, './src/.ice');
        this.storePath = storePath ? storePath : path.join(this.iceTemp, `./stores.${projectType}`);
        this.specifyStore = !!storePath;
        this.storeGenerator = this.getStore(storeType);
        this.prettierOptions = this.getPrettierOptions(rootDir);
        this.run = utils_1.debounce(this.run.bind(this), 1000);
        this.init(onGetWebpackConfig, applyMethod, command);
    }
    init(onGetWebpackConfig, applyMethod, command) {
        this.run();
        this.alias(onGetWebpackConfig);
        if (typeof this.storeGenerator.appendEntry === 'function') {
            this.storeGenerator.appendEntry((code, callback) => applyMethod('agreementAppend', [code, callback]));
        }
        if (command === 'start' || command === 'watch' || command === 'dev') {
            // 监听文件夹变化
            this.watch();
        }
    }
    alias(onGetWebpackConfig) {
        const { aliasName = 'ice/store' } = (this.storeGenerator || {});
        config_1.default({ aliasName, rootDir: this.rootDir, projectType: this.projectType, storePath: this.storePath });
        webpack_1.default({ aliasName, onGetWebpackConfig, storePath: this.storePath });
    }
    getStore(type) {
        if (type) {
            return stores_1.default[type];
        }
        // 从 package.json 文件中判断使用的 storeType
        const config = fs.readJSONSync(path.join(this.rootDir, './package.json'));
        const types = Object.values(stores_1.default).filter((store) => store.checkType && store.checkType(config));
        if (types.length === 0) {
            this.log.error('no install any state container, please set storeType in plugin options');
            return {};
        }
        if (types.length > 1) {
            this.log.error('install state container > 1, please set storeType in plugin options');
        }
        return types[0];
    }
    run() {
        if (this.specifyStore) {
            return;
        }
        this.log.info('\nBUILD-PLUGIN-STORE generator stores');
        const { hasIndex, models } = getModels_1.default(this.rootDir, this.iceTemp);
        if (hasIndex) {
            this.storePath = models[0].modelPath;
            this.specifyStore = true;
        }
        const source = generator_1.generator({ models, storeGenerator: this.storeGenerator, projectType: this.projectType, iceTemp: this.iceTemp, rootDir: this.rootDir });
        fs.ensureFileSync(this.storePath);
        fs.writeFileSync(this.storePath, prettier.format(source, this.prettierOptions));
    }
    watch() {
        const storesDir = path.join(this.rootDir, '/src/stores');
        chokidar.watch(storesDir).on('all', this.run);
    }
    getPrettierOptions(rootDir) {
        const defaultOptions = {
            parser: 'typescript',
            semi: true,
            singleQuote: true,
            tabWidth: 2,
            trailingComma: 'none',
        };
        try {
            const options = fs.readJSONSync(path.join(rootDir, './.prettierrc'));
            return {
                ...defaultOptions,
                ...options,
            };
        }
        catch (e) {
            return defaultOptions;
        }
    }
}
exports.default = PluginCore;
